#include <stdio.h>
#include <assert.h>

#include "systemc.h"

class Interface: virtual public sc_interface {

public:
  virtual int read(void) = 0;
  virtual void write(const int) = 0;
};

class Channel:
  public sc_channel,
  public Interface
{
public:
  Channel(sc_module_name name):
    sc_channel(name)
  {
      v = false;
      w = false;
  }

  ~Channel()
  { }

  int read(void)
  {
    if (!v) {
      w = true;
      wait(req);
      w = false;
    }

    int temp = tmpd;
    v = false;
    ack.notify(SC_ZERO_TIME);
    wait(ack);
    return temp;
  }

  void write(const int X)
  {
    tmpd = X;
    v = true;
    if (w) {
      req.notify(SC_ZERO_TIME);
    }
    wait(ack);
  }

private:
  sc_event req;
  sc_event ack;
  bool v, w;
  int tmpd;
};

// Sender
SC_MODULE(Sender)
{
  sc_port<Interface> port;

  SC_CTOR(Sender)
  {
    SC_THREAD(thread1);
  }

  void thread1(void)
  {
    std::cout << "start sender" << std::endl;
    for(int i=0; i<10; i++) {

      wait(1, SC_NS);
      port->write(i);
      wait(1, SC_NS);
    }
    std::cout << "end sender" << std::endl;
  }
};


// Receiver
SC_MODULE(Receiver)
{
  sc_port<Interface> port;

  SC_CTOR(Receiver)
  {
    SC_THREAD(thread1);
  }

  void thread1()
  {
    std::cout << "start receiver" << std::endl;
    double sum = 0;
    for(int i=0; i<10; i++) {

      int temp;
      wait(1, SC_NS);
      temp = port->read();
      wait(1, SC_NS);

      double temp1 = sin(temp);
      for(int j = 0; j < 10000000; j++) {

        temp1 = cos(temp1);
        temp1 = tan(temp1);
        temp1 = asin(temp1);
        temp1 = acos(temp1);
        temp1 = atan(temp1);
        temp1 = atan2(temp1, temp1);
        temp1 = cosh(temp1);
        temp1 = tanh(temp1);
        temp1 = asinh(temp1);
        temp1 = acosh(temp1);
        temp1 = atanh(temp1);
        temp1 = sinh(temp1);
        temp1 = atanh(temp1);
        temp1 = sinh(temp1);
      }

      sum += temp1;
    }
    sum++;
    std::cout << "end receiver" << std::endl;
  }
};


// Top
SC_MODULE(Top)
{
  // Thisis not good
  Channel **channel;
  Sender **sender;
  Receiver **receiver;

  int number_;

  Top(sc_module_name name, int number):
    sc_module(name),
    number_(number)
  {
    std::stringstream ss_c;
    std::stringstream ss_s;
    std::stringstream ss_r;

    receiver =  new Receiver*[number_];
    channel =  new Channel*[number_];
    sender =  new Sender*[number_];

    for(int i = 0 ; i < number_; i++) {
      ss_c << "c";
      ss_s << "s";
      ss_r << "r";

      sender[i] = new Sender(ss_s.str().c_str());
      receiver[i] = new Receiver(ss_r.str().c_str());
      channel[i] = new Channel(ss_c.str().c_str());
    }
  }

  void before_end_of_elaboration()
  {
    for(int i = 0 ; i < number_; i++) {
      sender[i]->port.bind(*channel[i]);
      receiver[i]->port.bind(*channel[i]);
    }
  }
};


int sc_main(int argc, char* argv[])
{
  if(argc == 1) {
    std::cout << "Error: Missing number of pairs." << std::endl;
    exit(1);
  }

  int number = 1;
  if(argc == 2) {
    number = strtol(argv[1], NULL ,10);
  }

  if(number < 1) {
    std::cout << "Error: Number must be bigger than 0." << std::endl;
    exit(1);
  } else {
    std::cout << "Number of pairs: " << number << std::endl;
  }

  Top top("top", number);
  sc_start();
  return 0;
}

// EOF

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
