// config.h: configuration file for NoC particle example
//
// modifications: (most recent first)
//
// 06/25/16 RD	guarded settings, enabling command-line override
// 06/23/16 RD	changed default settings for smaller golden files
// 06/22/16 RD	redesign for fixed number of particles in total
//		(i.e. not per tile); replaced N and EXTRA_SPACE
//		with total NUM_PARTICLES and MAX_PARTICLES per tile
// 06/20/16 RD	added a constant force (GRAVITY_X/Y)
// 06/20/16 RD	new configuration setting (more interesting movements)
// 06/18/16 RD	more cleanup
// 04/29/16 RD	conditional settings (may be set elsewhere)
// 04/29/16 RD	initial version (based on existing header)


// Definitions ////////////////////////////////////////////////////////////

#ifndef VERBOSITY
#define VERBOSITY 0	// quiet (no messages)
//#define VERBOSITY 1	// print progress messages
//#define VERBOSITY 2	// print more progress messages
//#define VERBOSITY 3	// print many progress messages
#endif

#ifndef DEBUG
//#define DEBUG		// print debugging messages
#endif

#ifndef NDEBUG
//#define NDEBUG	// turn off assertions
#endif

// SIMULATION configuration ///////////////////////////////////////////////

// select parallelism here
#ifndef NUM_DUT
#define NUM_DUT 	1
//#define NUM_DUT		4
//#define NUM_DUT	9
//#define NUM_DUT	16
#endif

// we assume a square tile layout
#ifndef NUM_ROWS
#define NUM_ROWS ((int)sqrt(NUM_DUT))
#endif
#ifndef NUM_COLS
#define NUM_COLS ((int)sqrt(NUM_DUT))
#endif
#define TILE_NO(row,col) ((row-1)*NUM_ROWS + col)

// set the number of particles simulated here
#ifndef NUM_PARTICLES	// total number of particles
#define NUM_PARTICLES	100
#endif
#ifndef MAX_PARTICLES	// particle storage per tile
#define MAX_PARTICLES	100
#endif

// select length of simulation here
#ifndef NSTEPS
#define NSTEPS		200
#endif

// pick none, one of the two, or both options here
//#define FALLING_INIT_ORIENTATION
//#define CENTER_INIT_ORIENTATION

// apply a constant force
#ifndef GRAVITY_X
#define GRAVITY_X	0
#endif
#ifndef GRAVITY_Y
#define GRAVITY_Y	-9.81*10
#endif

// turn on/off file saving of newcoords.txt file
#ifndef SAVE_FILE
#define SAVE_FILE  0
#endif

// adjust frequency of saving newcoords.txt here
#ifndef SAVEFREQ
#define SAVEFREQ 10	// save a frame every # steps
#endif

// particles tuned constants //////////////////////////////////////////////

#define density 0.0005
#define mass    0.01
#define cutoff  0.01
#define min_r   (cutoff/100)
#define dt      0.0005

// EOF config.h
