#ifndef DYNAMIC_INTERNAL_REPRESENTATION_H_INCLUDED_
#define DYNAMIC_INTERNAL_REPRESENTATION_H_INCLUDED_

#include <list>
#include <string>
#include <iostream>
#include <set>

#include "socket.h"
class DynamicHierarchicalChannel;

class DynamicObject {

public:
  std::string name_;
  int *address_;

  virtual void class_name() = 0;
};

class DynamicEventList: public DynamicObject {

public:
  std::string class_name_;
  std::list<int*> events_;

  virtual void class_name();
};

class DynamicPort: public DynamicObject {
public:
  virtual void class_name();

  void print(unsigned int indent);
};

//TLM2

class DynamicInitSocket: public DynamicObject {
public:
  virtual void class_name();

  void print(unsigned int indent);
};


class DynamicTargetSocket: public DynamicObject {
public:
  virtual void class_name();

  void print(unsigned int indent);
};


//end
class DynamicPointer: public DynamicObject {
public:
  virtual void class_name();

  void print(unsigned int indent);
};

class DynamicReference: public DynamicObject {
public:
  virtual void class_name();

  void print(unsigned int indent);
};

class DynamicVariable: public DynamicObject {
public:
  virtual void class_name();

  void print(unsigned int indent);
};

class DynamicEvent: public DynamicObject {
public:
  virtual void class_name();

  void print(unsigned int indent);
};

class DynamicPrimChannel: public DynamicObject {
public:
  std::string class_name_;

  virtual void class_name();

  void print(unsigned int indent);
};

class DynamicEventOrListElement: public DynamicObject {
public:
  std::string class_name_;
  int* event_;

  virtual void class_name();

  void print(unsigned int indent);
};

class DynamicEventOrList: public DynamicEventList {
public:

  virtual void class_name();

  void print(unsigned int indent);
};

class DynamicEventAndListElement: public DynamicObject {
public:
  std::string class_name_;
  int* event_;

  virtual void class_name();

  void print(unsigned int indent);
};

class DynamicEventAndList: public DynamicEventList {
public:

  virtual void class_name();

  void print(unsigned int indent);
};

class DynamicModule: public DynamicObject {

public:

  std::list<DynamicModule*> sub_modules_;
  std::list<DynamicPort*> ports_;
  std::list<DynamicInitSocket*> init_sockets_;
  std::list<DynamicTargetSocket*> target_sockets_;
  std::list<DynamicPointer*> pointers_;
  std::list<DynamicReference*> references_;
  std::list<DynamicVariable*> variables_;
  std::list<DynamicPrimChannel*> sub_channels_;
  std::list<DynamicHierarchicalChannel*> sub_hierarchical_channels_;
  std::list<DynamicEvent*> events_;
  std::list<DynamicEventAndList*> event_and_lists_;
  std::list<DynamicEventOrList*> event_or_lists_;

  virtual void class_name();

  void print_tree(unsigned int indent = 0);
};

class DynamicHierarchicalChannel: public DynamicModule {

public:

  virtual void class_name();

  void print_tree(unsigned int indent = 0);
};

/**
 * \fn       get_dynamic_object_of_variable(
              DynamicModule *root,
              std::string host_module_type_name,
              int host_module_instance_id,
              std::string reference_name,
              int &instance_counter)
 * \brief    This function searches for the mapped variable address of a
 *           reference in a given module with a specific instance id
 */
DynamicObject*
get_dynamic_object_of_variable(
    DynamicModule *root,
    std::string host_module_type_name,
    int host_module_instance_id,
    std::string reference_name,
    int &instance_counter);

/**
 * \fn       get_instance_id_of_hosting_module(
              DynamicModule *root,
              int *address_of_variable,
              std::string module_name,
              int &coutner)
 * \brief    This function determines to which (instance of a given) module
 *           (host module type name) a variable belongs to.
 *           The instance counter will provide the instance id. The starting
 *           value should be 0.
 */
bool
get_instance_id_of_hosting_module(
    DynamicModule *root,
    int *address_of_variable,
    std::string module_name,
    int &coutner);

/**
 * \fn       get_host_module_and_name_of_variable(DynamicModule *root, int *variable)
 * \brief    This function searchs the the variable address 'variable' in the
 *           tree. After finding the variable address the name of the variable
 *           and the hosting module will be returned.
 */
std::pair<DynamicModule*, std::string>
get_host_module_and_name_of_variable(DynamicModule *root, int *variable);

void
get_all_components_of_type(
    DynamicModule *root,
    std::string type_name,
    std::list<DynamicModule*> &modules);

// TODO Need to define input/output
DynamicModule*
get_host_module_of_member(DynamicModule *root, int *address_of_variable, int var_type = 0, std::string member_name = "");

/**
 * \fn    find_channel(DynamicModule *root, int *address_of_channel)
 * \brief Finds a channel instance in the dynamic tree via the address
 *
 */
DynamicObject* find_channel(DynamicModule *root, int *address_of_channel);

DynamicModule*
get_module_instance(
    DynamicModule *root,
    const std::string &name,
    int instance_id,
    int &counter);

int get_mapped_channel_id_of_port(DynamicPort *port);

void get_id(DynamicModule *root, std::string name, int *address, int &counter, bool &found);

DynamicTargetSocket* get_target_socket(DynamicModule *root, int *address_of_socket);
DynamicInitSocket* get_initiator_socket(DynamicModule *root, int *address_of_socket);
std::set<risc::Socket*> get_mapped_target_sockets(std::string init_socket_name, std::string host_module_name);
std::set<risc::Socket*> get_mapped_initiator_sockets(std::string target_socket_name, std::string host_module_name);
#endif /* DYNAMIC_INTERNAL_REPRESENTATION_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
