#include "dynamic_path_instance_mapper.h"

#include <stdlib.h>
#include <iostream>

#include "dynamic_internal_representation.h"
#include "hierarchical_channel.h"
#include "module.h"

#include "parser.dynamic_analysis.tab.h"
#include "lex.dynamic_analysis.h"
#include "internal_representation_helper.h"

int dynamic_analysisparse();

risc::DynamicPathInstanceMapper::DynamicPathInstanceMapper(
    std::string filename):
  filename_(filename),
  dynamic_root_(0)
{
  parse_dynamic_instance_tree();
  determine_lookup_table();
}

void
risc::DynamicPathInstanceMapper::parse_dynamic_instance_tree()
{
  FILE *file = fopen(filename_.c_str(), "r");
  if(file) {
//    std::cout << std::endl;
//    std::cout << "Reading dynamic input file " << filename_ << std::endl;
    dynamic_analysisin = file;
    dynamic_analysisparse();

    dynamic_root_ = ast_root;
    global_variable_addresses_ = global_variable_addresses;

  } else {
    std::cout << "Cannot open file: " << filename_ << std::endl;
    exit(1);
  }
}

void
risc::DynamicPathInstanceMapper::determine_lookup_table()
{
  InstanceTree root;
  InstanceTree::Element first;
  root.push_back(first);

  // Here, we are incrementing the instance counter for the root element
  // manually
  Module *module = find_module_definition(ast_root->name_);
  HierarchicalChannel *channel = NULL;

  if(!module) {
    channel = find_hierarchical_channel_definition(ast_root->name_);
  }

  if(module) {
    module->instance_counter_++;
  } else {
    channel->instance_counter_++;
  }

  if(module) {
    if(max_instances_ < module->instance_counter_) {
      max_instances_ = module->instance_counter_;
    }
  } else {
    if(max_instances_ < channel->instance_counter_) {
      max_instances_ = channel->instance_counter_;
    }
  }

  InstanceTree::Element second(NULL,NULL);

  if(module) {
    second.first = module;
  } else {
    second.first = channel;
  }

  root.push_back(second);

  PathWithID temp(root, 0);
  path_to_id_mapping_.push_back(temp);

  traverse_tree(root, ast_root);
}

void
risc::DynamicPathInstanceMapper::traverse_tree(
    InstanceTree root,
    DynamicModule *module)
{
  for(std::list<DynamicModule*>::iterator
      iter  = module->sub_modules_.begin();
      iter != module->sub_modules_.end();
      iter++) {

    risc::InstanceTree::Element element;
    Module *module = find_module_definition((*iter)->name_);
    HierarchicalChannel *channel = NULL;

    if(module) {
      element.first = module;
    } else {
      channel = find_hierarchical_channel_definition((*iter)->name_);
      element.first = channel;
    }

    element.second = NULL;
    root.push_back(element);

    PathWithID temp(root, 0);
    if(module) {
      temp.second = module->instance_counter_;
    } else {
      temp.second = channel->instance_counter_;
    }

    path_to_id_mapping_.push_back(temp);

    if(module) {
      module->instance_counter_++;
    } else {
      channel->instance_counter_++;
    }

    traverse_tree(root, *iter);

    root.pop_back();

    if(module) {
      if(max_instances_ < module->instance_counter_) {
        max_instances_ = module->instance_counter_;
      }
    } else {
      if(max_instances_ < channel->instance_counter_) {
        max_instances_ = channel->instance_counter_;
      }
    }
  }

  for(std::list<DynamicHierarchicalChannel*>::iterator
      iter  = module->sub_hierarchical_channels_.begin();
      iter != module->sub_hierarchical_channels_.end();
      iter++) {

    risc::InstanceTree::Element element;
    Module *module = find_module_definition((*iter)->name_);
    HierarchicalChannel *channel = NULL;

    if(module) {
      element.first = module;
    } else {
      channel = find_hierarchical_channel_definition((*iter)->name_);
      element.first = channel;
    }

    element.second = NULL;
    root.push_back(element);

    PathWithID temp(root, 0);
    if(module) {
      temp.second = module->instance_counter_;
    } else {
      temp.second = channel->instance_counter_;
    }

    path_to_id_mapping_.push_back(temp);

    if(module) {
      module->instance_counter_++;
    } else {
      channel->instance_counter_++;
    }

    traverse_tree(root, *iter);

    root.pop_back();

    if(module) {
      if(max_instances_ < module->instance_counter_) {
        max_instances_ = module->instance_counter_;
      }
    } else {
      if(max_instances_ < channel->instance_counter_) {
        max_instances_ = channel->instance_counter_;
      }
    }
  }
}

DynamicModule* risc::DynamicPathInstanceMapper::get_dynamic_tree_root()
{
  return dynamic_root_;
}

bool
risc::DynamicPathInstanceMapper::perform_dynamic_analysis()
{
  return true;
}

int*
risc::DynamicPathInstanceMapper::get_global_variable_address(
    std::string variable_name)
{
  for(std::list<std::pair<std::string, int*> >::iterator
      iter  = global_variable_addresses_.begin();
      iter != global_variable_addresses_.end();
      iter++) {

    if(variable_name == (*iter).first) {
      return (*iter).second;
    }
  }

  return 0;
}

std::string
risc::DynamicPathInstanceMapper::get_global_variable_name(int *variable_address)
{
  for(std::list<std::pair<std::string, int*> >::iterator
      iter  = global_variable_addresses_.begin();
      iter != global_variable_addresses_.end();
      iter++) {

    if(variable_address == (*iter).second) {
      return (*iter).first;
    }
  }

  return "";
}


std::string
risc::DynamicPathInstanceMapper::translate_type_name(std::string gnu_type_name)
{

  if(gnu_type_name == "N7sc_core7sc_fifoIiEE") {
    return "L1784R";
  }

  if(gnu_type_name == "N7sc_core7sc_fifoIbEE") {
    return "L1791R";
  }

  if(gnu_type_name == "N7sc_core7sc_fifoIsEE") {
    return "L1798R";
  }

  if(gnu_type_name == "N7sc_core7sc_fifoIcEE") {
    return "L1805R";
  }

  if(gnu_type_name == "N7sc_core7sc_fifoIlEE") {
    return "L1812R";
  }

  if(gnu_type_name == "N7sc_core7sc_fifoIxEE") {
    return "L1819R";
  }

  if(gnu_type_name == "N7sc_core7sc_fifoIfEE") {
    return "L1826R";
  }

  if(gnu_type_name == "N7sc_core7sc_fifoIdEE") {
    return "L1833R";
  }

  if(gnu_type_name == "N7sc_core7sc_fifoIeEE") {
    return "L1840R";
  }

  return "";
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
