#include "socket.h"

/*
#include "inSocket.h"
#include "inoutSocket.h"
#include "outSocket.h"
*/

#include "instance.h"
#include "module_instance.h"

int risc::Socket::socket_id_counter_=0;

risc::Socket::Socket(SgVariableDefinition *ast_node, Class *parent, Type type):
  Object(ast_node, type),
  parent_(parent),
  id_(Socket::socket_id_counter_),
  is_end_(false)
{ 
  ++Socket::socket_id_counter_;
}

std::string
risc::Socket::get_ast_type_name()
{
  return get_ast_type()->unparseToString();//.substr(17);
}

void risc::Socket::helper_get_mapped_sockets(
  std::set<risc::Socket*> &mapped_sockets, bool is_forward)
{
  // Strategy (refer to risc::Port::get_mapped_channels)
  // One Socket can be mapped to many other sockets
  // because we are on the level of a socket definition.
  //
  // So, as a starting point we have a Socket.
  // First, we identify the surrounding module(channel?) where it is defined(declared?).
  // The module(channel?) definition stores a list of all its instances.
  // For each instance we look up the mapping of the Socket.
  // Note that because a socket must be finally mapped to another socket,
  // and vice versa, we use recursive calls of this helper function to find the 
  // destination sockets.
  //
  // is_forward specifies the mapping.
  // if true, then is from initiator to target
  // if false, then is from target to initiator
  // get_mapped_sockets(..) in InitiatorSocket and TargetSocket calls this helper function
  //
  
  // Now we simply assume that all the sockets are in Modules
  Module *parent = dynamic_cast<Module*>(parent_);

  for(std::list<Instance*>::iterator
      instance_iter  = parent->instances_.begin();
      instance_iter != parent->instances_.end();
      instance_iter ++) {

    ModuleInstance *module_instance = NULL;
    if((*instance_iter)->type_ == ModuleInstanceType) {
      module_instance = dynamic_cast<ModuleInstance*>(*instance_iter);
    }

    //socket maps to socket
    std::map<Socket*, Socket*>::iterator mapped;

    if(module_instance) {

      if(is_forward)
      {
        //this means we want to follow the forward mapping
        mapped = module_instance->socket_mapping_.find(this);
      
        if(mapped == module_instance->socket_mapping_.end()){
          std::cout 
            << "this socket is not bound in this parent instance, and checks next instance of the module" << std::endl;
          continue;
        }
      }
      else
      {
        //this means we want to follow the backward mapping
        mapped = module_instance->backward_socket_mapping_.find(this);
      
        if(mapped == module_instance->backward_socket_mapping_.end()){
          std::cout 
            << "this socket is not bound in this parent instance, and checks next instance of the module" << std::endl;
          continue;
        }
      }
      
      //if mapped socket type is an end
      //then it is a found and teminates
      //otherwise, it means we need to go further
      if(((*mapped).second)->is_end_)
        mapped_sockets.insert((*mapped).second);
      else
        ((*mapped).second)->helper_get_mapped_sockets(mapped_sockets, is_forward);
    }
  }
}


/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
