#ifndef SOCKET_H_INCLUDED_
#define SOCKET_H_INCLUDED_

#include "rose.h"

//why did Tim include module.h?
//because the parent is a class (module or channel I guess)
#include "module.h"
#include "object.h"

namespace risc {

class Socket: public Object {

public:

  /**
   * \brief default constructor
   */
  explicit
  Socket(SgVariableDefinition *ast_node, Class *parent, Type type = SocketType);

  /**
   * \brief Get function for the name of the ast type of the socket
   * \return Name as std::string
   */
  virtual std::string get_ast_type_name();

  Class *parent_;

  static int socket_id_counter_;
  int id_;
  bool is_end_;
protected:
  /**
   * \fn helper_get_mapped_sockets(std::set<risc::Socket*> &mapped_sockets, bool is_forward)
   * \brief This function determines all potential mapped sockets to this socket.
   *        If this socket is an InitiatoreSocketType, the mapped are all 
   *        TargetSocketType.
   *        And vice versa.
   *        type specifies the type of socket that should be collected into 
   *        mapped_sockets
   * CAUTION:
   * If you want to get the specific mapped socket,
   * you have to use the path instance mapper.
   */
  void helper_get_mapped_sockets(std::set<risc::Socket*> &mapped_sockets, bool is_forward);

  //TODO: currently, we dont support PSG
  /*
  * after loading psg, this flag of the loaded port is set to true
  *
  */
  //bool bounded;
private:

  /**
   * \brief Disabled copy constructor
   */
  Socket(const Socket &p);
};

typedef std::vector<Socket*> SocketVector;
typedef std::vector<Socket*>::iterator SocketVectorIter;
typedef std::vector<Socket*>::const_iterator SocketVectorConstIter;

} // end of namespace risc

#endif /* SOCKET_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
