#ifndef FUNCTION_ANNOTATION_H_INCLUDED_
#define FUNCTION_ANNOTATION_H_INCLUDED_

#include <iostream>
#include <string>

enum WAIT_CONSTRUCT {
  NO_WAIT = 0,
  UNCONDITIONAL_WAIT = 1,
  CONDITIONAL_WAIT = 2,
  LOOPED_WAIT = 3,
  WORST_WAIT = 4
};

struct AnnotatedTime {

  AnnotatedTime():
    units_(0),
    magnitude_(0),
    event_notification_(false),
    sc_zero_time_(false)
  { }

  /**
   * \brief The time unit, NS, PS, so on
   */
  int units_;

  /**
   * \brief The time
   */
  long long magnitude_;

  /**
   * \brief If contains notification
   */
  bool event_notification_;

  /**
   * \brief If the segment waits for zero time
   */
  bool sc_zero_time_;
};

bool operator==(const AnnotatedTime &lhs, const AnnotatedTime &rhs);

struct WaitAnnotation {

  WAIT_CONSTRUCT wait_type_;
  AnnotatedTime *annotated_time_;
};

struct FunctionAnnotation {

  FunctionAnnotation(
      char *function_name,
      bool is_conflict_free,
      WAIT_CONSTRUCT wait_type,
      AnnotatedTime *annotated_time):
    function_name_(function_name),
    is_conflict_free_(is_conflict_free),
    wait_type_(wait_type),
    annotated_time_(annotated_time)
  { }


  /**
   * \brief The function name
   */
  std::string function_name_;

  /**
   * \brief Whether the function contains no conflict
   */
  bool is_conflict_free_;

  /**
   * \brief Type of waiting inside
   */
  WAIT_CONSTRUCT wait_type_;

  /**
   * \brief If waiting for time, what the time is
   */
  AnnotatedTime *annotated_time_;

  friend std::ostream& operator<<(std::ostream& os, const FunctionAnnotation& fa)
  {
    os << "Function name:    " << fa.function_name_ << std::endl
       << "Is conflict free: " << fa.is_conflict_free_ << std::endl
       << "Wait type:        ";

    switch(fa.wait_type_) {

      case NO_WAIT:
        os << "NO_WAIT" << std::endl;
        break;
      case UNCONDITIONAL_WAIT:
        os << "UNCONDITIONAL_WAIT" << std::endl;
        break;
      case CONDITIONAL_WAIT:
        os << "CONDITIONAL_WAIT" << std::endl;
        break;
      case LOOPED_WAIT:
        os << "LOOPED_WAIT" << std::endl;
        break;
      case WORST_WAIT:
        os << "WORST_WAIT" << std::endl;
        break;
    }

    if(fa.annotated_time_->event_notification_) {
      os << "Time:             event" << std::endl;

    } else if(fa.annotated_time_->sc_zero_time_) {
      os << "Time:             sc-zero_time" << std::endl;

    } else {
      os << "Time:             " << fa.annotated_time_->units_ << ", ";

      switch(fa.annotated_time_->magnitude_) {

        case 1:
          os << "SC_FS" << std::endl;
          break;
        case 1000:
          os << "SC_PS" << std::endl;
          break;
        case 1000000:
          os << "SC_NS" << std::endl;
          break;
        case 1000000000:
          os << "SC_US" << std::endl;
          break;
        case 1000000000000l:
          os << "SC_MS" << std::endl;
          break;
        case 1000000000000000l:
          os << "SC_SEC" << std::endl;
          break;
      }
    }
    return os;
  }
};

#endif /* FUNCTION_ANNOTATION_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
