#ifndef VARIABLE_HELPER_H_INCLUDED_
#define VARIABLE_HELPER_H_INCLUDED_

#include "rose.h"

#include "../segment_graph/port_call_path.h"

namespace risc {

namespace tools {

/**
 * \fn is_reference_variable(SgVariableSymbol *var_symbol)
 * \brief    This function return true if the given variable symbol is a
 *           reference
 */
bool is_reference_variable(SgVariableSymbol *var_symbol);

/**
 * \fn is_pointer_variable(SgVariableSymbol *var_symbol)
 * \brief    This function return true if the given variable symbol is a
 *           pointer
 */
bool is_pointer_variable(SgVariableSymbol *var_symbol);

/**
 * \fn is_global_variable(SgVariableSymbol *var_symbol)
 * \brief    This function return true if the given variable symbol is a
 *           global variable
 */
bool is_global_variable(SgVariableSymbol *var_symbol);

/**
 * \fn is_local_variable(SgVariableSymbol *var_symbol)
 * \brief    This function return true if the given variable symbol is a
 *           local variable
 */
bool is_local_variable(SgVariableSymbol *var_symbol);

/**
 * \fn is_member_variable(SgVariableSymbol *var_symbol)
 * \brief    This function return true if the given variable symbol is a
 *           member variable
 */
bool is_member_variable(SgVariableSymbol *var_symbol);

/**
 * \fn is_static_variable(SgVariableSymbol *var_symbol)
 * \brief    This function return true if the given variable symbol is a
 *           static variable
 */
bool is_static_variable(SgVariableSymbol *var_symbol);

/**
 * \fn is_namespace_variable(SgVariableSymbol *var_symbol)
 * \brief    This function return true if the given variable symbol is a
 *           namespace variable
 */
bool is_namespace_variable(SgVariableSymbol *var_symbol);

/**
 * \fn is_function_parameter_variable(SgVariableSymbol *var_symbol)
 * \brief    This function return true if the given variable symbol is a
 *           function variable
 */
bool is_function_parameter_variable(SgVariableSymbol *var_symbol);

/**
 * \fn is_member_of(SgVariableSymbol *is, SgSymbol *of)
 * \brief    This function returns true if a given variable symbol is defined
 *           'in' another variable symbol. For instance 'class Foo { int i;}'
 *           'i' is defined in 'Foo'
 */
bool is_member_of(SgVariableSymbol *is, SgSymbol *of);

/**
 * \fn is_loop_variable(SgVariableSymbol *var_symbol)
 * \brief    This function return true if the given variable symbol is a
 *           loop variable
 */
bool is_loop_variable(SgVariableSymbol *var_symbol);

/**
 * \fn is_stack_variable(SgVariableSymbol *var_symbol)
 * \brief    This function return true if the given variable symbol is a
 *           variable which will be stored on the stack
 */
bool is_stack_variable(SgVariableSymbol *var_symbol);

/**
 * \fn is_data_segment_variable(SgVariableSymbol *var_symbol)
 * \brief    This function return true if the given variable symbol is a
 *           variable which will be stored on the stack
 */
bool is_data_segment_variable(SgVariableSymbol *var_symbol);

/**
 * \fn analyze_conflict_free(SgVariableSymbol *symbol)
 * \brief    This function checks if a variable is annoated with a pragma
 *           as conflict free
 */
void analyze_conflict_free(SgVariableSymbol *symbol);

/**
 * \fn is_channel_variable(SgVariableSymbol *symbol)
 * \brief    Returns true when the variable is declared in a channel
 */
bool is_channel_variable(SgVariableSymbol *symbol);

/**
 * \fn is_sc_signal(SgVariableSymbol *symbol)
 * \brief    Returns true if the variable is of type sc_signal
 */
bool is_sc_signal(SgVariableSymbol *symbol);

/**
 * \fn is_socket_variable(risc::sg::SymbolWithPath swp)
 * \brief    Returns true when the symbolwithpath has a pcp_.port_ attribute
 */
bool is_socket_variable(risc::sg::SymbolWithPath swp);

}; // end of namespace tools

}; // end of namespace risc

#endif /* VARIABLE_HELPER_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
