#include "tlm.h"                                    // TLM headers
#include "tlm_utils/simple_initiator_socket.h"      // TLM simple initiator socket
#include "tlm_utils/simple_target_socket.h"
#include <string>
#include "systemc.h"
using namespace sc_core;
using namespace std;


/*------------------initiator---------------------*/
class lt_initiator :  public sc_core::sc_module 
{
public:
    lt_initiator( sc_core::sc_module_name name);
  	tlm::tlm_sync_enum  custom_nb_transport_bw( 
		tlm::tlm_generic_payload  &trans,
		tlm::tlm_phase            &phase,
		sc_core::sc_time          &delay_time
  	);
    SC_HAS_PROCESS(lt_initiator);
   
    tlm_utils::simple_initiator_socket<lt_initiator> initiator_socket;
 	int init_data;
}; 

lt_initiator::lt_initiator( sc_module_name name )
: sc_module           (name)                      // initialize module name
, initiator_socket    ("initiator_socket")        // initiator socket
{                
	init_data = 42;
	initiator_socket.register_nb_transport_bw(this, &lt_initiator::custom_nb_transport_bw);               
}

tlm::tlm_sync_enum                                          
lt_initiator::custom_nb_transport_bw( 
	tlm::tlm_generic_payload  &trans,
	tlm::tlm_phase            &phase,
	sc_core::sc_time          &delay_time          
)
{
	wait(2,SC_NS);
	unsigned char* ptr = trans.get_data_ptr();
	int len = trans.get_data_length();
	memcpy(&init_data, ptr, len);

	wait(1,SC_NS);
	std::cout << "init_data = " << init_data << std::endl;

	return tlm::TLM_ACCEPTED;
}

/*------------------target---------------------*/
class lt_target : public sc_core::sc_module           	
{
public:	  
 
	lt_target( sc_core::sc_module_name   module_name);
  	void target_thread (void);
	SC_HAS_PROCESS(lt_target);
	int tar_data;
	tlm_utils::simple_target_socket<lt_target>  target_socket;

};

lt_target::lt_target( sc_core::sc_module_name module_name )
: sc_module (module_name)
, target_socket ("target_socket")
{
	tar_data = 25;
	SC_THREAD(target_thread); 
}


void lt_target::target_thread(void)   ///< initiator thread
{  
	tlm::tlm_generic_payload trans_target; 
	trans_target.set_data_ptr( reinterpret_cast<unsigned char*>(&tar_data));
	trans_target.set_data_length(4);
	sc_time delay_tar = SC_ZERO_TIME;       
	tlm::tlm_phase phase  = tlm::END_RESP;

	target_socket->nb_transport_bw(trans_target, phase, delay_tar);
} 

/*------------------top---------------------*/

class lt_top : public sc_core::sc_module                   
{
public:
		
	lt_top ( sc_core::sc_module_name name); 
	lt_target               m_target_1;          
	lt_initiator            m_initiator_1;  
};



lt_top::lt_top( sc_core::sc_module_name name )
: sc_core::sc_module(name)
, m_target_1("m_target_1")
, m_initiator_1("m_initiator_1")
{
  std::cout << "in top.cpp" << std::endl;
  m_initiator_1.initiator_socket(m_target_1.target_socket);
}

int sc_main(int   argc, char  *argv[] )
{
  lt_top top("top");                    // instantiate a exmaple top module

  sc_core::sc_start();                  // start the simulation

  return 0;                             // return okay status
}
