#include "tlm.h"                                    // TLM headers
#include "tlm_utils/simple_initiator_socket.h"      // TLM simple initiator socket
#include "tlm_utils/simple_target_socket.h"
#include <string>
#include "systemc.h"
using namespace sc_core;
using namespace std;

/************** initiator class ********************/
class lt_initiator :  public sc_core::sc_module 
{
public:
    lt_initiator( sc_core::sc_module_name name);
  
  	void initiator_thread (void);                    
  
    SC_HAS_PROCESS(lt_initiator);
   
    tlm_utils::simple_initiator_socket<lt_initiator> m_init_socket;
 
    int data;
}; 


lt_initiator::lt_initiator( sc_module_name name )
: sc_module           (name)                      // initialize module name
, m_init_socket    ("init_socket1")        // initiator socket
{                
	SC_THREAD(initiator_thread);                  
}

void lt_initiator::initiator_thread(void)   ///< initiator thread
{  
	tlm::tlm_generic_payload trans; 
	data = 10;
	sc_time delay = SC_ZERO_TIME;       // Create delay objects 
	std::cout << "entered initiator_thread" << std::endl;
	m_init_socket->b_transport(trans, delay);

} 


/************** target class ********************/
class lt_target : public sc_core::sc_module           	
{
public:	  
 
	lt_target( sc_core::sc_module_name   module_name);

	void target_b_transport( 
		tlm::tlm_generic_payload  &gp,
		sc_core::sc_time          &delay_time
  	);

	SC_HAS_PROCESS(lt_target);
	tlm_utils::simple_target_socket<lt_target>  m_target_socket;

	std::string              m_target_memory;
};


lt_target::lt_target( sc_core::sc_module_name module_name )
: sc_module (module_name)
, m_target_socket ("target_socket1")
, m_target_memory ("Hello World")
{
  m_target_socket.register_b_transport(this, &lt_target::target_b_transport);
}

void                                        
lt_target::target_b_transport( 
	tlm::tlm_generic_payload  &gp,
	sc_core::sc_time          &delay_time             // delay time 
)
{
 
  int length = gp.get_data_length();
  wait(1,SC_NS);
  std::cout << "target_b_transport called" << std::endl;
    
}


/************** bus class ********************/
class lt_bus : public sc_core::sc_module           	
{
public:	  
 
	lt_bus( sc_core::sc_module_name   module_name);

	void bus_b_transport1( 
		tlm::tlm_generic_payload  &gp,
		sc_core::sc_time          &delay_time
  	);
	void bus_b_transport2( 
		tlm::tlm_generic_payload  &gp,
		sc_core::sc_time          &delay_time
  	);

	SC_HAS_PROCESS(lt_bus);
	
	tlm_utils::simple_target_socket<lt_bus>  m_bus_tar_socket1;
	tlm_utils::simple_target_socket<lt_bus>  m_bus_tar_socket2;

	tlm_utils::simple_initiator_socket<lt_bus>  m_bus_init_socket1;
	tlm_utils::simple_initiator_socket<lt_bus>  m_bus_init_socket2;

};


lt_bus::lt_bus( sc_core::sc_module_name module_name )
: sc_module (module_name)
, m_bus_tar_socket1 ("bus_tar_socket1")
, m_bus_tar_socket2 ("bus_tar_socket2")
, m_bus_init_socket1 ("bus_init_socket1")
, m_bus_init_socket2 ("bus_init_socket2")
{
  m_bus_tar_socket1.register_b_transport(this, &lt_bus::bus_b_transport1);
  m_bus_tar_socket2.register_b_transport(this, &lt_bus::bus_b_transport2);
}

void                                        
lt_bus::bus_b_transport1( 
	tlm::tlm_generic_payload  &gp,
	sc_core::sc_time          &delay_time             // delay time 
)
{  
  std::cout << "bus_b_transport1 called" << std::endl;
  m_bus_init_socket1->b_transport(gp, delay_time);

}


void                                        
lt_bus::bus_b_transport2( 
	tlm::tlm_generic_payload  &gp,
	sc_core::sc_time          &delay_time             // delay time 
)
{  
  std::cout << "bus_b_transport2 called" << std::endl;
  m_bus_init_socket2->b_transport(gp, delay_time);
    
}

/************** top class ********************/
class lt_top : public sc_core::sc_module                   
{
public:
		
	lt_top ( sc_core::sc_module_name name); 
	lt_target               m_lt_target_1; 
	lt_target               m_lt_target_2;          
	lt_initiator            m_initiator_1;          
	lt_initiator            m_initiator_2;          
	lt_bus 					m_bus_1;
};

lt_top::lt_top( sc_core::sc_module_name name )
: sc_core::sc_module(name)
, m_lt_target_1("m_lt_target_1")
, m_lt_target_2("m_lt_target_2")
, m_initiator_1("m_initiator_1")
, m_initiator_2("m_initiator_2")
, m_bus_1("m_bus_1")
{
  std::cout << "initialization in top module" << std::endl;
  m_initiator_1.m_init_socket(m_bus_1.m_bus_tar_socket1);
  m_initiator_2.m_init_socket(m_bus_1.m_bus_tar_socket2);

  m_bus_1.m_bus_init_socket1(m_lt_target_1.m_target_socket);
  m_bus_1.m_bus_init_socket2(m_lt_target_2.m_target_socket);
}


/************** main ********************/
int sc_main(int   argc, char  *argv[] )
{
  lt_top top("top");                    // instantiate a exmaple top module

  sc_core::sc_start();                  // start the simulation

  return 0;                             // return okay status
}
