// event_and_list3.cc:
//
// 06/26/18 RD	modified from event_and_list2.cc (added timing checks)

#include <assert.h>
#include "systemc.h"

SC_MODULE(top) {
	sc_event e1;
	sc_event e2;
        sc_event_and_list e1ANDe2;
	int n1, n2, w1, w2, w3;
        sc_time tn1,tn2,tw1,tw2,tw3;

	SC_CTOR(top) {
		n1 = n2 = w1 = w2 = w3 = 0;
                tn1 = SC_ZERO_TIME;
                tn2 = SC_ZERO_TIME;
                tw1 = SC_ZERO_TIME;
                tw2 = SC_ZERO_TIME;
                tw3 = SC_ZERO_TIME;
		SC_THREAD(notifier1);
		SC_THREAD(notifier2);
		SC_THREAD(waiter1);
		SC_THREAD(waiter2);
		SC_THREAD(waiter3);
	}

	void before_end_of_elaboration() {
		e1ANDe2 &= e1;
		e1ANDe2 &= e2;
	}

	void notifier1() {
		wait(5,SC_NS);
		std::cout << "notifier 1 sending notification!\n";
		e1.notify(SC_ZERO_TIME);
		n1 = 1;
                tn1 = sc_time_stamp();
	}

	void notifier2() {
		wait(6,SC_NS);
		std::cout << "notifier 2 sending notification!\n";
		e2.notify(SC_ZERO_TIME);
		n2 = 1;
                tn2 = sc_time_stamp();
	}

	void waiter1() {
		wait(e1);
		std::cout << "waiter 1 received notification!\n";
		w1 = 1;
                tw1 = sc_time_stamp();
	}

	void waiter2() {
		wait(e2);
		std::cout << "waiter 2 received notification!\n";
		w2 = 1;
                tw2 = sc_time_stamp();
	}

	void waiter3() {
		wait(e1ANDe2);
		std::cout << "waiter 3 received notification!\n";
		w3 = 1;
                tw3 = sc_time_stamp();
	}
};

int sc_main(int argc, char** argv) {
	top t("t");
        const sc_time tn1(5, SC_NS);
        const sc_time tn2(6, SC_NS);
        const sc_time tw1(5, SC_NS);
        const sc_time tw2(6, SC_NS);
        const sc_time tw3(6, SC_NS);

	sc_start();
        cout << t.n1 << " at " << t.tn1.to_string() << ", "
             << t.n2 << " at " << t.tn2.to_string() << ", "
             << t.w1 << " at " << t.tw1.to_string() << ", "
             << t.w2 << " at " << t.tw2.to_string() << ", "
             << t.w3 << " at " << t.tw3.to_string() << endl;
	assert(t.n1 == 1);
	assert(t.n2 == 1);
	assert(t.w1 == 1);
	assert(t.w2 == 1);
	assert(t.w3 == 1);
        assert(t.tn1 == tn1);
        assert(t.tn2 == tn2);
        assert(t.tw1 == tw1);
        assert(t.tw2 == tw2);
        assert(t.tw3 == tw3);
	return 0;
}

/* ex: set softtabstop=8 tabstop=8 shiftwidth=8 expandtab: */
