// time_resolution10ps.cc: time resolution test, 10ps version
// RD, 12/10/18

#include <stdio.h>
#include <assert.h>

#include "systemc.h"

#define ITERATIONS      5

SC_MODULE(Worker1ps)
{
    unsigned Count;
    unsigned State;

    void main(void)
    {
        sc_time ExpectedDelay(ITERATIONS*0, SC_PS);

        State = 1;
	for(int i=0; i<ITERATIONS; i++)
	{
	    wait(1, SC_PS); // expected to be rounded down to 0ps
	    Count++;
	}
        State = 2;
	assert(Count == ITERATIONS);
        cout << "Worker1ps delay:    " << sc_time_stamp() << "\n";
        assert(sc_time_stamp() == ExpectedDelay);
        assert(sc_time_stamp().to_string() == "0 s");
    }

    SC_CTOR(Worker1ps)
    {
	Count = 0;
	State = 0;
	SC_HAS_PROCESS(Worker1ps);
	SC_THREAD(main);
    }
};

SC_MODULE(Worker500fs)
{
    unsigned Count;
    unsigned State;

    void main(void)
    {
        sc_time ExpectedDelay(ITERATIONS*0, SC_PS);

        State = 1;
	for(int i=0; i<ITERATIONS; i++)
	{
	    wait(500, SC_FS); // expected to be rounded down to 0ps
	    Count++;
	}
        State = 2;
	assert(Count == ITERATIONS);
        cout << "Worker500fs delay:  " << sc_time_stamp() << "\n";
        assert(sc_time_stamp() == ExpectedDelay);
        assert(sc_time_stamp().to_string() == "0 s");
    }

    SC_CTOR(Worker500fs)
    {
	Count = 0;
	State = 0;
	SC_HAS_PROCESS(Worker500fs);
	SC_THREAD(main);
    }
};

SC_MODULE(Worker6ps)
{
    unsigned Count;
    unsigned State;

    void main(void)
    {
        sc_time ExpectedDelay(ITERATIONS*10, SC_PS);

        State = 1;
	for(int i=0; i<ITERATIONS; i++)
	{
	    wait(6, SC_PS); // expected to be rounded up to 10ps
	    Count++;
	}
        State = 2;
	assert(Count == ITERATIONS);
        cout << "Worker6ps delay:    " << sc_time_stamp() << "\n";
        assert(sc_time_stamp() == ExpectedDelay);
        assert(sc_time_stamp().to_string() == "50 ps");
    }

    SC_CTOR(Worker6ps)
    {
	Count = 0;
	State = 0;
	SC_HAS_PROCESS(Worker6ps);
	SC_THREAD(main);
    }
};

SC_MODULE(TOP)
{
    Worker1ps w1;
    Worker500fs w2;
    Worker6ps w3;

    SC_CTOR(TOP)
    : w1("w1"), w2("w2"), w3("w3")
    {
    }
};

TOP top("top");

int sc_main(int argc, char* argv[])
{
    printf("time_resolution10ps starts\n");
    sc_set_time_resolution(10, SC_PS);
    cout << "chosen resolution:  " << sc_get_time_resolution() << "\n";

    sc_time ExpectedTotalTime(ITERATIONS*10, SC_PS);
    sc_time ExpectedResolution(10, SC_PS);

    assert(sc_get_time_resolution() == ExpectedResolution);
    cout << "sc_max_time:        " << sc_max_time() << "\n";
    sc_start();
    assert(top.w1.Count == ITERATIONS);
    assert(top.w2.Count == ITERATIONS);
    assert(top.w3.Count == ITERATIONS);
    assert(top.w1.State == 2);
    assert(top.w2.State == 2);
    assert(top.w3.State == 2);
    cout << "total simulated time: " << sc_time_stamp() << " ";
    assert(sc_time_stamp() == ExpectedTotalTime);
    assert(sc_time_stamp().to_string() == "50 ps");
    printf("= %g seconds (internal %llu = %g)\n",
        sc_time_stamp().to_seconds(),
        sc_time_stamp().value(),
        sc_time_stamp().to_double());
    assert(sc_time_stamp().to_seconds() == 5e-11);
    assert(sc_time_stamp().value() == 5ull);
    assert(sc_time_stamp().to_double() == 5.0);
    printf("time_resolution10ps done.\n");
    return 0;
}

// EOF

/* ex: set softtabstop=8 tabstop=8 shiftwidth=8 expandtab: */
